package	jp.co.sds_corp.util.calc;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.HashMap;
import java.util.Map;

/**
* 計算式を処理します。
*/
public class CalcExpression
{
	String exp;
	ICalcCallback	callback;
	CalcCompiler	compiler = new CalcCompiler();
	CalcNode		topNode;
	Map<String,Class<?>>	mapCls = new HashMap<String,Class<?>>();
	Map<String,Object>	mapObj = new HashMap<String,Object>();

	/**
	* 計算式を処理するインスタンスを生成します。
	* @param	exp		計算式
	*/
	public CalcExpression( String exp ) throws SyntaxException {
		this.exp = exp;
		include( "Math", Math.class );
	}

	/**
	* 計算時に使用するコールバックオブジェクトを登録します。
	* @param	callback	コールバックオブジェクト
	*/
	public void setCallback( ICalcCallback callback ) {
		this.callback = callback;
	}

	/**
	* 計算に使用されるクラスを登録します。
	* @param	name	名前
	* @param	cls		計算に使用されるクラス
	*/
	public void include( String name, Class<?> cls ) {
		mapCls.put( name, cls );
	}

	/**
	* 計算に使用されるクラスを登録します。
	* @param	cls		計算に使用されるクラス
	*/
	public void include( Class<?> cls ) {
		include( cls.getName(), cls );
	}

	/**
	* 計算に使用されるインスタンスを登録します。
	* @param	name	名前
	* @param	obj		計算に使用されるインスタンス
	*/
	public void include( String name, Object obj ) {
		include( name, obj.getClass() );
		mapObj.put( name, obj );
	}

	/**
	* 計算に使用されるインスタンスを登録します。
	* @param	obj		計算に使用されるインスタンス
	*/
	public void include( Object obj ) {
		include( obj.getClass().getName(), obj.getClass() );
	}

	/**
	* 計算結果を取得します。
	*/
	public BigDecimal calc() throws Exception {
		if ( topNode == null ) {
			compile();
		}
		return calc( topNode );
	}

	/**
	* 計算式をコンパイルします。
	*/
	public void compile() throws Exception {
		topNode = compiler.compile( exp );
	}

	/**
	* 計算結果を取得します。
	*/
	protected BigDecimal calc( CalcNode node ) throws Exception {
		BigDecimal ret = null;
		switch ( node.getType()) {
		case CalcNode.TYPE_VALUE:
			ret = ((DecimalNode )node).getValue();
			break;
		case CalcNode.TYPE_LITERAL:
			String name = ((StringNode )node).getString();
			ret = getField( name );
			if ( ret == null ) {
				if ( callback == null ) {
					throw( new SyntaxException( "callback not found. var=[" + name + "]" ));
				}
				ret = callback.getValue( name );
			}
			break;
		case CalcNode.TYPE_OPERATOR:
			Operator ope = ((OperatorNode )node).getOperator();
			BigDecimal prm2 = calc( node.getChild2() );
			BigDecimal prm1 = calc( node.getChild1() );
			ret = ope.calc( prm1, prm2 );
			break;
		case CalcNode.TYPE_FUNCTION:
			Function func = ((FunctionNode )node).getFunction();
			ret = callMethod( func );
			if ( ret == null ) {
				if ( callback == null ) {
					throw( new SyntaxException( "callback not found. function=["
												+ func.getName() + "]" ));
				}
				CalcNode[] param = func.getParam();
				BigDecimal[] bprm = new BigDecimal[param.length];
				for ( int i = 0; i < param.length; i++ ) {
					bprm[i] = calc( param[i] );
				}
				ret = callback.callFunction( func.getName(), bprm );
			}
			break;
		}
		return ret;
	}

	/**
	* 登録されているクラスから、フィールドの値を取得します。
	* @param	name	フィールド名
	*/
	public BigDecimal getField( String name ) throws Exception {
		int idx = name.lastIndexOf( "." );
		if ( idx < 0 ) {
			return null;
		}
		String clsName = name.substring( 0, idx );
		Class<?> cls = (Class<?> )mapCls.get( clsName );
		if ( cls == null ) {
			return null;
		}
		String fldName = name.substring( idx + 1 );
		Field fld = null;
		try {
			fld = cls.getField( fldName );
		} catch ( NoSuchFieldException nfe ) {
		}
		if ( fld == null ) {
			return null;
		}
		Object obj = mapObj.get( clsName );
		BigDecimal ret = new BigDecimal( fld.getDouble( obj ));
		return ret;
	}

	/**
	* 登録されているクラスのメソッドを呼び出します。
	* @param	func	メソッド
	*/
	public BigDecimal callMethod( Function func ) throws Exception {
		int idx = func.getName().lastIndexOf( "." );
		if ( idx < 0 ) {
			return null;
		}
		String clsName = func.getName().substring( 0, idx );
		Class<?> cls = (Class<?> )mapCls.get( clsName );
		if ( cls == null ) {
			clsName = "java.lang." + clsName;
			cls = (Class<?> )mapCls.get( clsName );
			if ( cls == null ) {
				return null;
			}
		}
		String mtdName = func.getName().substring( idx + 1 );
		Method[] arrMtd = cls.getMethods();
		Method mtd = null;
		for ( int i = 0; i < arrMtd.length; i++ ) {
			if ( arrMtd[i].getName().equals( mtdName ) &&
				arrMtd[i].getParameterTypes().length == func.getParam().length ) {
				mtd = arrMtd[i];
				break;
			}
		}
		if ( mtd == null ) {
			return null;
		}
		Object[] param = new Object[func.getParam().length];
		Class<?>[] types = mtd.getParameterTypes();
		for ( int i = 0; i < param.length; i++ ) {
			BigDecimal bprm = calc( func.getParam()[i] );
			if ( types[i] == double.class ) {
				param[i] = Double.valueOf( bprm.doubleValue());
			} else
			if ( types[i] == float.class ) {
				param[i] = Float.valueOf( bprm.floatValue());
			} else
			if ( types[i] == int.class ) {
				param[i] = Integer.valueOf( bprm.intValue());
			} else
			if ( types[i] == long.class ) {
				param[i] = Long.valueOf( bprm.longValue());
			} else
			if ( types[i] == BigDecimal.class ) {
				param[i] = bprm;
			} else
			if ( types[i] == BigInteger.class ) {
				param[i] = BigInteger.valueOf( bprm.longValue() );
			}
		}
		Object obj = mapObj.get( clsName );
		Object ret = mtd.invoke( obj, param );
		BigDecimal dec = null;
		if ( mtd.getReturnType() == double.class ) {
			dec = new BigDecimal( ((Double )ret).doubleValue() );
		} else
		if ( mtd.getReturnType() == float.class ) {
			dec = new BigDecimal( ((Float )ret).doubleValue() );
		} else
		if ( mtd.getReturnType() == int.class ) {
			dec = new BigDecimal( ((Integer )ret).doubleValue() );
		} else
		if ( mtd.getReturnType() == long.class ) {
			dec = new BigDecimal( ((Long )ret).doubleValue() );
		} else
		if ( mtd.getReturnType() == BigDecimal.class ) {
			dec = (BigDecimal )ret;
		}
		return dec;
	}

	/**
	* 計算式の情報を文字列で返します。
	*/
	public String toString() {
		if ( topNode == null ) {
			return "";
		}
		return topNode.toString();
	}

}
