package	jp.co.sds_corp.util.config;

import java.io.Serializable;
import java.util.Iterator;
import java.util.List;

/**
* 設定ファイルの情報を保持するオブジェクトのためのinterfaceです。
* <p>
* XML(DOMConfiguration)を例に言えば、このオブジェクトが特定のノード
* または特定のノードの属性の集合を表し、
* 子要素の内容または属性値を取得するメソッド群を持つことになります。
* </p><p>
* defaultValueパラメタを取らないメソッドは、値が存在しない場合に
* UnspecifiedPropertyExceptionをthrowします。<br>
* 省略可能な設定情報の取得は、defaultValueを持つメソッドを用いてください。
* </p>
*/
public interface IConfiguration extends Serializable
{
    /**
     * 親コンテキストを表すIConfigurationを返します。
     */
    IConfiguration getParent();

    /**
     * 現在着目している要素の子要素の内、指定名称を持つ最初の子要素を
     * 表すIConfigurationを返します。
     * @param name 子要素名
     * @return 指定名称の子要素を表すIConfigurationオブジェクト
     */
    IConfiguration getChild( String name );

    /**
     * 現在着目している要素の子要素を列挙します。
     * <P>
     * 列挙される内容は各子要素を表すIConfigurationです。
     * </P>
     * @return 各子要素を表すIConfigurationオブジェクトのイテレータ
     */
    Iterator<IConfiguration> getChilds();

    /**
     * 現在着目している要素の子要素を列挙します。
     * <P>
     * 列挙される内容は各子要素を表すIConfigurationです。
     * </P>
     * @return 各子要素を表すIConfigurationオブジェクトのイテレータ
     */
    List<IConfiguration> listChild();

    /**
     * 現在着目している要素の子要素の内、指定名称を持つ子要素を列挙します。
     * <P>
     * 列挙されるのは各子要素を表すIConfigurationです。
     * </P>
     * @param name 子要素名
     * @return 指定名称の子要素を表すIConfigurationオブジェクトのイテレータ
     */
    Iterator<IConfiguration> getChilds( String name );

    /**
     * 現在着目している要素の子要素の内、指定名称を持つ子要素を列挙します。
     * <P>
     * 列挙されるのは各子要素を表すIConfigurationです。
     * </P>
     * @param name 子要素名
     * @return 指定名称の子要素を表すIConfigurationオブジェクトのイテレータ
     */
    List<IConfiguration> listChild( String name );

    /**
     * 現在着目している要素の子要素の内、指定名称と指定属性を持つ子要素を取得します。
     * @param name 子要素名
     * @param attrName 属性名
     * @param attrValue 属性値
     * @return 指定名称の子要素を表すIConfigurationオブジェクト
     */
    IConfiguration getChild( String name, String attrName, String attrValue );

    /**
     * 現在着目しているノードの属性群を表すIConfigurationを返します。
     * <p>
     * XMLにおいて&lt;foo name=value /&gt;要素を表すIConfigurationオブジェクト
     * fooが存在する時、属性値の取得は
     * </p>
     * <pre><code>foo.getAttributes().get("name")</code></pre>
     * <p>の様になります。</p>
     * @return 現在の要素の属性を表すIConfigurationオブジェクト
     */
    IConfiguration getAttributes();

    /**
     * 現在着目しているノード名を返します。
     */
    String getName();

    /**
     * 現在着目しているノードの、指定された子要素の内容をintとして返します。
     * @param key 子要素名
     * @return 子要素の内容をparseInt()したもの。
     * @exception UnspecifiedPropertyException
     *            keyに対応する値が取得できなかった(子要素または内容がない)
     * @exception ConfigurationFormatException 内容がintでなかった
     */
    int getInteger( String key )
            throws UnspecifiedPropertyException, ConfigurationFormatException;
    int getInteger( String key, int defaultValue )
            throws ConfigurationFormatException;

    /**
     * 現在着目しているノードの、指定された子要素の内容をfloatとして返します。
     * @param key 子要素名
     * @return 子要素の内容をparseFloat()したもの。
     * @exception UnspecifiedPropertyException
     *            keyに対応する値が取得できなかった(子要素または内容がない)
     * @exception ConfigurationFormatException 内容がfloatでなかった
     */
    float getFloat( String key )
            throws UnspecifiedPropertyException, ConfigurationFormatException;
    float getFloat( String key, float defaultValue )
            throws ConfigurationFormatException;

    /**
     * 現在着目しているノードの、指定された子要素の内容をbooleanと
     * して返します。Booleanクラスの仕様に従い、フォーマットのエラーは
     * 検出しません(したほうがいいかもしれないけど取り合えず)。
     * 値が"true"以外の場合はfalseを返します。
     * なお、値が存在しない場合はUnspecifiedPropertyExceptionをthrowします。
     * @param key 子要素名
     * @return 子要素の内容をnew Boolean(str).booleanValue()したもの。
     * @exception UnspecifiedPropertyException
     *            keyに対応する値が取得できなかった(子要素または内容がない)
     */
    boolean getBoolean( String key ) throws UnspecifiedPropertyException;
    boolean getBoolean( String key, boolean defaultValue );


    /**
     * 現在着目しているノードの、指定された子要素の内容を文字列として返します。
     * @param key 子要素名
     * @return 子要素の内容文字列。
     * @exception UnspecifiedPropertyException
     *            keyに対応する値が取得できなかった(子要素または内容がない)
     */
    String get( String key ) throws UnspecifiedPropertyException;
    String get( String key, String defaultValue );
    /**
     * 自身のノードが表現する値(内容)を返します。
     */
    String get() throws UnspecifiedPropertyException;

	public IConfiguration addChild( IConfiguration child );
	public IConfiguration addChild( String name );
	public IConfiguration addChild( String name, String value );
	public void set( String value );
	public void setAttribute( String name, String value );
}
