package jp.co.sds_corp.util.math;


/**
* 関数f(x)の解を、2分法によって求めます。
*/
public class Bisection
{
	/**
	* 関数f(x)
	*/
	public interface Function {
		public double of(double x) throws UtilMathException;
	}

	public static final int EXCEPTION_THROW = 1;
	public static final int MIN_X_RETURN    = 0;

	private static boolean isSameSign(double a, double b) {
		return (a > 0) == (b > 0);
	}

	/**
	* 2分法により解を求めます。
	* @param	f		f(x)
	* @param	a		範囲の最小値
	* @param	b		範囲の最大値
	* @param	step	解が求められなかった時に移動する値
	* @param	tolerance	許容限度
	* @param	m		移動する側 1:最小値側から
	* @param	mode	エラー時の動作 1:Exception
	* @param	eps		
	*/
	public static double solve(Function f, double a, double b, double step,
				double tolerance, int m, int mode, double eps)
		throws UtilMathException
	{
		boolean ok = true;

		double minF = 0.0;
		double minX = 0.0;

		if (tolerance < 0) {
			tolerance = 0;
		}
		if (b < a) {
			double swap = a;
			a = b;
			b = swap;
		}
		double fa = f.of(a);
		if (fa == 0) {
			return a;
		}
		double fb = f.of(b);
		if (fb == 0) {
			return b;
		}

		if (fa > fb) {
			minF = fa;
			minX = a;
		} else {
			minF = fb;
			minX = b;
		}

		if (isSameSign(fa, fb)) {
			if (m == 1) {
				a += step;
			} else {
				b -= step;
			}
			while (true) {
				if (m == 1) { 
					fa = f.of(a);
					if (Math.abs(fa) < Math.abs(minF)) {
						minF = fa;
						minX = a;
					}
				} else {
					fb = f.of(b);
					if (Math.abs(fb) < Math.abs(minF)) {
						minF = fb;
						minX = b;
					}
				}

				if (b <= a) {
					ok = false;
					break;
				}
				if (isSameSign(fa, fb)) {
					if (m == 1) {
						a += step;
					} else {
						b -= step;
					}
					continue;
				} else {
					ok = true;
					break;
				}
			}
			if (!ok) {
				if (mode == EXCEPTION_THROW) { 
					throw new UtilMathException(
						UtilMathException.BISECTION_EXCEPTION, 
						"区間の両端で関数値が同負号です。" + a + ":" + b,
						new Exception());
				} else {
					if (minF < 0) {
						throw new UtilMathException(
							UtilMathException.BISECTION_EXCEPTION, 
							"区間の両端で関数値が同負号です。" + a + ":" + b,
							new Exception());
					} else {
						if (minF > eps) {
							throw new UtilMathException(
								UtilMathException.BISECTION_EXCEPTION, 
								"区間の両端で関数値が同負号です。" + a +":"+ b,
								new Exception());
						}
					}
				}
				return minX;
			}
		}

		for (int i=0; ; i++) {
			double c = (a + b) / 2;
			if (c - a <= tolerance || b - c <= tolerance) {
				return c;
			}
			double fc = f.of(c);
			if (fc == 0) {
				return c;
			}
			if (isSameSign(fc, fa)) {
				a = c; fa = fc; 
			} else {
				b = c; fb = fc; 
			}
		}
	}
}
